@@ -8,12 +8,14 @@ except ImportError:
 import sys
 import tempfile
 from os.path import getsize
+from os import unlink
 
+from transitions.extensions.nesting import NestedState
 from transitions.extensions import MachineFactory
-from transitions.extensions.nesting import NestedState as State
+
 from unittest import skipIf
-from .test_core import TestTransitions as TestsCore
-from .utils import Stuff
+from .test_core import TestTransitions, TestCase
+from .utils import Stuff, DummyModel
 
 try:
     from unittest.mock import MagicMock
@@ -26,41 +28,33 @@ try:
 except ImportError:  # pragma: no cover
     pgv = None
 
-state_separator = State.separator
+
+default_separator = NestedState.separator
 
 
 class Dummy(object):
     pass
 
 
-class TestTransitions(TestsCore):
+test_states = ['A', 'B',
+               {'name': 'C', 'children': ['1', '2', {'name': '3', 'children': ['a', 'b', 'c']}]}, 'D', 'E', 'F']
+
+
+class TestNestedTransitions(TestTransitions):
 
     def setUp(self):
-        states = ['A', 'B', {'name': 'C', 'children': ['1', '2', {'name': '3', 'children': ['a', 'b', 'c']}]},
-                  'D', 'E', 'F']
-        machine_cls = MachineFactory.get_predefined(nested=True)
-        self.stuff = Stuff(states, machine_cls)
-
-    def tearDown(self):
-        State.separator = state_separator
-        pass
+        self.states = test_states
+        self.machine_cls = MachineFactory.get_predefined(nested=True)
+        self.state_cls = NestedState
+        self.stuff = Stuff(self.states, self.machine_cls)
 
     def test_add_model(self):
         model = Dummy()
         self.stuff.machine.add_model(model, initial='E')
 
-    def test_function_wrapper(self):
-        from transitions.extensions.nesting import FunctionWrapper
-        mo = MagicMock
-        f = FunctionWrapper(mo, ['a', 'long', 'path', 'to', 'walk'])
-        f.a.long.path.to.walk()
-        self.assertTrue(mo.called)
-        with self.assertRaises(Exception):
-            f.a.long.path()
-
     def test_init_machine_with_hella_arguments(self):
         states = [
-            State('State1'),
+            self.state_cls('State1'),
             'State2',
             {
                 'name': 'State3',
@@ -80,13 +74,17 @@ class TestTransitions(TestsCore):
         self.assertEqual(s.message, 'Hello World!')
 
     def test_init_machine_with_nested_states(self):
+        State = self.state_cls
         a = State('A')
         b = State('B')
-        b_1 = State('1', parent=b)
-        b_2 = State('2', parent=b)
+        b_1 = State('1')
+        b_2 = State('2')
+        b.add_substate(b_1)
+        b.add_substates([b_2])
         m = self.stuff.machine_cls(states=[a, b])
-        self.assertEqual(b_1.name, 'B{0}1'.format(state_separator))
-        m.to("B{0}1".format(state_separator))
+        self.assertEqual(m.states['B'].states['1'], b_1)
+        m.to("B{0}1".format(State.separator))
+        self.assertEqual(m.state, "B{0}1".format(State.separator))
 
     def test_property_initial(self):
         # Define with list of dictionaries
@@ -105,6 +103,7 @@ class TestTransitions(TestsCore):
         self.assertEqual(m.initial, 'initial')
 
     def test_transition_definitions(self):
+        State = self.state_cls
         states = ['A', 'B', {'name': 'C', 'children': ['1', '2', '3']}, 'D']
         # Define with list of dictionaries
         transitions = [
@@ -131,13 +130,32 @@ class TestTransitions(TestsCore):
         m.sprint()
         self.assertEqual(m.state, 'D')
 
+    def test_nested_definitions(self):
+        separator = self.state_cls.separator
+        state = {
+            'name': 'B',
+            'children': ['1', '2'],
+            'transitions': [['jo', '1', '2']],
+            'initial': '2'
+        }
+        m = self.stuff.machine_cls(initial='A', states=['A', state],
+                                   transitions=[['go', 'A', 'B'], ['go', 'B{0}2'.format(separator),
+                                                                   'B{0}1'.format(separator)]])
+        self.assertTrue(m.is_A())
+        m.go()
+        self.assertEqual(m.state, 'B{0}2'.format(separator))
+        m.go()
+        self.assertEqual(m.state, 'B{0}1'.format(separator))
+        m.jo()
+
     def test_transitioning(self):
         s = self.stuff
         s.machine.add_transition('advance', 'A', 'B')
         s.machine.add_transition('advance', 'B', 'C')
         s.machine.add_transition('advance', 'C', 'D')
         s.machine.add_transition('reset', '*', 'A')
-        self.assertEqual(len(s.machine.events['reset'].transitions['C%s1' % State.separator]), 1)
+        self.assertEqual(len(s.machine.events['reset'].transitions), 6)
+        self.assertEqual(len(s.machine.events['reset'].transitions['C']), 1)
         s.advance()
         self.assertEqual(s.state, 'B')
         self.assertFalse(s.is_A())
@@ -159,6 +177,7 @@ class TestTransitions(TestsCore):
         self.assertEqual(s.state, 'C')
 
     def test_multiple_add_transitions_from_state(self):
+        State = self.state_cls
         s = self.stuff
         s.machine.add_transition(
             'advance', 'A', 'B', conditions=['this_fails'])
@@ -180,134 +199,76 @@ class TestTransitions(TestsCore):
         self.assertEqual(m.state, 'B')
 
     def test_add_custom_state(self):
+        State = self.state_cls
         s = self.stuff
         s.machine.add_states([{'name': 'E', 'children': ['1', '2']}])
-        s.machine.add_state('3', parent='E')
+        s.machine.add_state('E%s3' % State.separator)
         s.machine.add_transition('go', '*', 'E%s1' % State.separator)
         s.machine.add_transition('walk', '*', 'E%s3' % State.separator)
         s.machine.add_transition('run', 'E', 'C{0}3{0}a'.format(State.separator))
         s.go()
-        self.assertEqual(s.state, 'E{0}1'.format(State.separator))
+        self.assertEqual('E{0}1'.format(State.separator), s.state)
         s.walk()
-        self.assertEqual(s.state, 'E{0}3'.format(State.separator))
+        self.assertEqual('E{0}3'.format(State.separator), s.state)
         s.run()
-        self.assertEqual(s.state, 'C{0}3{0}a'.format(State.separator))
+        self.assertEqual('C{0}3{0}a'.format(State.separator), s.state)
+
+    def test_add_nested_state(self):
+        m = self.machine_cls(states=['A'], initial='A')
+        m.add_state('B{0}1{0}a'.format(self.state_cls.separator))
+        self.assertIn('B', m.states)
+        self.assertIn('1', m.states['B'].states)
+        self.assertIn('a', m.states['B'].states['1'].states)
+
+        with self.assertRaises(ValueError):
+            m.add_state(m.states['A'])
 
     def test_enter_exit_nested_state(self):
+        State = self.state_cls
         mock = MagicMock()
 
         def callback():
             mock()
         states = ['A', 'B', {'name': 'C', 'on_enter': callback, 'on_exit': callback,
-                             'children': [{'name': '1', 'on_exit': callback}, '2', '3']}, 'D']
+                             'children': [{'name': '1', 'on_enter': callback, 'on_exit': callback}, '2', '3']},
+                            {'name': 'D', 'on_enter': callback, 'on_exit': callback}]
         transitions = [['go', 'A', 'C{0}1'.format(State.separator)],
                        ['go', 'C', 'D']]
         m = self.stuff.machine_cls(states=states, transitions=transitions, initial='A')
         m.go()
         self.assertTrue(mock.called)
-        self.assertEqual(mock.call_count, 1)
+        self.assertEqual(2, mock.call_count)
         m.go()
         self.assertTrue(m.is_D())
-        self.assertEqual(mock.call_count, 3)
-
-    def test_state_change_listeners(self):
-        s = self.stuff
-        s.machine.add_transition('advance', 'A', 'C%s1' % State.separator)
-        s.machine.add_transition('reverse', 'C', 'A')
-        s.machine.add_transition('lower', 'C%s1' % State.separator, 'C{0}3{0}a'.format(State.separator))
-        s.machine.add_transition('rise', 'C%s3' % State.separator, 'C%s1' % State.separator)
-        s.machine.add_transition('fast', 'A', 'C{0}3{0}a'.format(State.separator))
-        s.machine.on_enter_C('hello_world')
-        s.machine.on_exit_C('goodbye')
-        s.machine.on_enter('C{0}3{0}a'.format(State.separator), 'greet')
-        s.machine.on_exit('C%s3' % State.separator, 'meet')
-        s.advance()
-        self.assertEqual(s.state, 'C%s1' % State.separator)
-        self.assertEqual(s.message, 'Hello World!')
-        s.lower()
-        self.assertEqual(s.state, 'C{0}3{0}a'.format(State.separator))
-        self.assertEqual(s.message, 'Hi')
-        s.rise()
-        self.assertEqual(s.state, 'C%s1' % State.separator)
-        self.assertTrue(s.message.startswith('Nice to'))
-        s.reverse()
-        self.assertEqual(s.state, 'A')
-        self.assertTrue(s.message.startswith('So long'))
-        s.fast()
-        self.assertEqual(s.state, 'C{0}3{0}a'.format(State.separator))
-        self.assertEqual(s.message, 'Hi')
-        s.to_A()
-        self.assertEqual(s.state, 'A')
-        self.assertTrue(s.message.startswith('So long'))
-
-    def test_enter_exit_nested(self):
-        s = self.stuff
-        s.machine.add_transition('advance', 'A', 'C{0}3'.format(State.separator))
-        s.machine.add_transition('reverse', 'C', 'A')
-        s.machine.add_transition('lower', ['C{0}1'.format(State.separator),
-                                           'C{0}3'.format(State.separator)], 'C{0}3{0}a'.format(State.separator))
-        s.machine.add_transition('rise', 'C%s3' % State.separator, 'C%s1' % State.separator)
-        s.machine.add_transition('fast', 'A', 'C{0}3{0}a'.format(State.separator))
-        for state in s.machine.states.values():
-            state.on_enter.append('increase_level')
-            state.on_exit.append('decrease_level')
-
-        s.advance()
-        self.assertEqual(s.state, 'C%s3' % State.separator)
-        self.assertEqual(s.level, 2)
-        self.assertEqual(s.transitions, 3)  # exit A; enter C,3
-        s.lower()
-        self.assertEqual(s.state, 'C{0}3{0}a'.format(State.separator))
-        self.assertEqual(s.level, 3)
-        self.assertEqual(s.transitions, 4)  # enter a
-        s.rise()
-        self.assertEqual(s.state, 'C%s1' % State.separator)
-        self.assertEqual(s.level, 2)
-        self.assertEqual(s.transitions, 7)  # exit a, 3; enter 1
-        s.reverse()
-        self.assertEqual(s.state, 'A')
-        self.assertEqual(s.level, 1)
-        self.assertEqual(s.transitions, 10)  # exit 1, C; enter A
-        s.fast()
-        self.assertEqual(s.state, 'C{0}3{0}a'.format(State.separator))
-        self.assertEqual(s.level, 3)
-        self.assertEqual(s.transitions, 14)  # exit A; enter C, 3, a
-        s.to_A()
-        self.assertEqual(s.state, 'A')
-        self.assertEqual(s.level, 1)
-        self.assertEqual(s.transitions, 18)  # exit a, 3, C; enter A
-        s.to_A()
-        self.assertEqual(s.state, 'A')
-        self.assertEqual(s.level, 1)
-        self.assertEqual(s.transitions, 20)  # exit A; enter A
-        if State.separator in '_':
-            s.to_C_3_a()
-        else:
-            s.to_C.s3.a()
-        self.assertEqual(s.state, 'C{0}3{0}a'.format(State.separator))
-        self.assertEqual(s.level, 3)
-        self.assertEqual(s.transitions, 24)  # exit A; enter C, 3, a
+        self.assertEqual(5, mock.call_count)
+        m.to_C()
+        self.assertEqual(7, mock.call_count)
+        m.to('C{0}1'.format(State.separator))
+        self.assertEqual(8, mock.call_count)
+        m.to('C{0}2'.format(State.separator))
+        self.assertEqual(9, mock.call_count)
 
     def test_ordered_transitions(self):
+        State = self.state_cls
         states = [{'name': 'first', 'children': ['second', 'third', {'name': 'fourth', 'children': ['fifth', 'sixth']},
                                                  'seventh']}, 'eighth', 'ninth']
         m = self.stuff.machine_cls(states=states)
         m.add_ordered_transitions()
-        self.assertEqual(m.state, 'initial')
+        self.assertEqual('initial', m.state)
         m.next_state()
-        self.assertEqual(m.state, 'first')
+        self.assertEqual('first', m.state)
         m.next_state()
         m.next_state()
-        self.assertEqual(m.state, 'first{0}third'.format(State.separator))
+        self.assertEqual('first{0}third'.format(State.separator), m.state)
         m.next_state()
         m.next_state()
-        self.assertEqual(m.state, 'first{0}fourth{0}fifth'.format(State.separator))
+        self.assertEqual('first{0}fourth{0}fifth'.format(State.separator), m.state)
         m.next_state()
         m.next_state()
-        self.assertEqual(m.state, 'first{0}seventh'.format(State.separator))
+        self.assertEqual('first{0}seventh'.format(State.separator), m.state)
         m.next_state()
         m.next_state()
-        self.assertEqual(m.state, 'ninth')
+        self.assertEqual('ninth', m.state)
 
         # Include initial state in loop
         m = self.stuff.machine_cls('self', states)
@@ -330,7 +291,7 @@ class TestTransitions(TestsCore):
         self.assertEqual(m.state, 'first{0}second'.format(State.separator))
 
     def test_pickle(self):
-        import sys
+        print("separator", self.state_cls.separator)
         if sys.version_info < (3, 4):
             import dill as pickle
         else:
@@ -373,45 +334,14 @@ class TestTransitions(TestsCore):
         self.assertEqual(m.before_change.call_count, 2)
         self.assertEqual(m.after_change.call_count, 2)
 
-    def test_with_custom_separator(self):
-        State.separator = '.'
-        self.setUp()
-        self.test_enter_exit_nested()
-        self.setUp()
-        self.test_state_change_listeners()
-        self.test_nested_auto_transitions()
-        State.separator = '.' if sys.version_info[0] < 3 else u'↦'
-        self.setUp()
-        self.test_enter_exit_nested()
-        self.setUp()
-        self.test_state_change_listeners()
-        self.test_nested_auto_transitions()
-
-    def test_with_slash_separator(self):
-        State.separator = '/'
-        self.setUp()
-        self.test_enter_exit_nested()
-        self.setUp()
-        self.test_state_change_listeners()
-        self.test_nested_auto_transitions()
-        self.setUp()
-        self.test_ordered_transitions()
-
-    def test_nested_auto_transitions(self):
-        s = self.stuff
-        s.to_C()
-        self.assertEqual(s.state, 'C')
-        state = 'C{0}3{0}a'.format(State.separator)
-        s.to(state)
-        self.assertEqual(s.state, state)
-        # backwards compatibility check (can be removed in 0.7)
-        self.assertEqual(s.state, state)
-
     def test_example_one(self):
+        State = self.state_cls
         State.separator = '_'
         states = ['standing', 'walking', {'name': 'caffeinated', 'children': ['dithering', 'running']}]
         transitions = [['walk', 'standing', 'walking'],
                        ['stop', 'walking', 'standing'],
+                       ['drink', 'caffeinated_dithering', '='],
+                       ['drink', 'caffeinated', 'caffeinated_dithering'],
                        ['drink', '*', 'caffeinated'],
                        ['walk', 'caffeinated', 'caffeinated_running'],
                        ['relax', 'caffeinated', 'standing']]
@@ -423,6 +353,10 @@ class TestTransitions(TestsCore):
         machine.drink()  # coffee time
         machine.state
         self.assertEqual(machine.state, 'caffeinated')
+        machine.drink()  # again!
+        self.assertEqual(machine.state, 'caffeinated_dithering')
+        machine.drink()  # and again!
+        self.assertEqual(machine.state, 'caffeinated_dithering')
         machine.walk()   # we have to go faster
         self.assertEqual(machine.state, 'caffeinated_running')
         machine.stop()   # can't stop moving!
@@ -434,37 +368,11 @@ class TestTransitions(TestsCore):
         machine.to_caffeinated_running()  # auto transition fast track
         machine.on_enter_caffeinated_running('callback_method')
 
-    def test_example_two(self):
-        State.separator = '.' if sys.version_info[0] < 3 else u'↦'
-        states = ['A', 'B',
-                  {'name': 'C', 'children': ['1', '2',
-                                             {'name': '3', 'children': ['a', 'b', 'c']}]
-                   }]
-
-        transitions = [
-            ['reset', 'C', 'A'],
-            ['reset', 'C%s2' % State.separator, 'C']  # overwriting parent reset
-        ]
-
-        # we rely on auto transitions
-        machine = self.stuff.machine_cls(states=states, transitions=transitions, initial='A')
-
-        machine.to_B()  # exit state A, enter state B
-        machine.to_C()  # exit B, enter C
-        machine.to_C.s3.a()  # enter C↦a; enter C↦3↦a;
-        self.assertEqual(machine.state, 'C{0}3{0}a'.format(State.separator))
-        machine.to('C{0}2'.format(State.separator))  # exit C↦3↦a, exit C↦3, enter C↦2
-        machine.reset()  # exit C↦2; reset C has been overwritten by C↦3
-        self.assertEqual(machine.state, 'C')
-        machine.reset()  # exit C, enter A
-        self.assertEqual(machine.state, 'A')
-
     def test_multiple_models(self):
         class Model(object):
             pass
         s1, s2 = Model(), Model()
-        m = MachineFactory.get_predefined(nested=True)(model=[s1, s2], states=['A', 'B', 'C'],
-                                                       initial='A')
+        m = self.stuff.machine_cls(model=[s1, s2], states=['A', 'B', 'C'], initial='A')
         self.assertEqual(len(m.models), 2)
         m.add_transition('advance', 'A', 'B')
         self.assertNotEqual(s1.advance, s2.advance)
@@ -481,38 +389,69 @@ class TestTransitions(TestsCore):
         m = self.stuff.machine_cls(states=states, initial='A')
 
     def test_intial_state(self):
+        separator = self.state_cls.separator
         states = [{'name': 'A', 'children': ['1', '2'], 'initial': '2'},
                   {'name': 'B', 'initial': '2',
                    'children': ['1', {'name': '2', 'initial': 'a',
                                       'children': ['a', 'b']}]}]
         transitions = [['do', 'A', 'B'],
-                       ['do', 'B{0}2'.format(state_separator),
-                        'B{0}1'.format(state_separator)]]
+                       ['do', 'B{0}2'.format(separator),
+                        'B{0}1'.format(separator)]]
         m = self.stuff.machine_cls(states=states, transitions=transitions, initial='A')
-        self.assertEqual(m.state, 'A{0}2'.format(state_separator))
+        self.assertEqual(m.state, 'A{0}2'.format(separator))
         m.do()
-        self.assertEqual(m.state, 'B{0}2{0}a'.format(state_separator))
+        self.assertEqual(m.state, 'B{0}2{0}a'.format(separator))
         self.assertTrue(m.is_B(allow_substates=True))
         m.do()
-        self.assertEqual(m.state, 'B{0}1'.format(state_separator))
+        self.assertEqual(m.state, 'B{0}1'.format(separator))
+        m = self.stuff.machine_cls(states=states, transitions=transitions, initial='B{0}2{0}b'.format(separator))
+        self.assertTrue('B{0}2{0}b'.format(separator), m.state)
 
     def test_get_triggers(self):
+        seperator = self.state_cls.separator
         states = ['standing', 'walking', {'name': 'caffeinated', 'children': ['dithering', 'running']}]
         transitions = [
             ['walk', 'standing', 'walking'],
             ['go', 'standing', 'walking'],
             ['stop', 'walking', 'standing'],
-            {'trigger': 'drink', 'source': '*', 'dest': 'caffeinated_dithering',
+            {'trigger': 'drink', 'source': '*', 'dest': 'caffeinated{0}dithering'.format(seperator),
              'conditions': 'is_hot', 'unless': 'is_too_hot'},
-            ['walk', 'caffeinated_dithering', 'caffeinated_running'],
+            ['walk', 'caffeinated{0}dithering'.format(seperator), 'caffeinated{0}running'.format(seperator)],
             ['relax', 'caffeinated', 'standing']
         ]
 
         machine = self.stuff.machine_cls(states=states, transitions=transitions, auto_transitions=False)
-        trans = machine.get_triggers('caffeinated{0}dithering'.format(state_separator))
+        trans = machine.get_triggers('caffeinated{0}dithering'.format(seperator))
         self.assertEqual(len(trans), 3)
         self.assertTrue('relax' in trans)
 
+    def test_get_nested_transitions(self):
+        seperator = self.state_cls.separator
+        states = ['A', {'name': 'B', 'children': ['1', '2', {'name': '3', 'children': ['a', 'b'],
+                                                             'transitions': [['inner', 'a', 'b'],
+                                                                             ['inner', 'b', 'a']]}],
+                        'transitions': [['mid', '1', '1'],
+                                        ['mid', '2', '3'],
+                                        ['mid', '3', '1'],
+                                        ['mid2', '2', '3'],
+                                        ['mid_loop', '1', '1']]}]
+        transitions = [['outer', 'A', 'B'],
+                       ['outer', ['A', 'B'], 'C']]
+        machine = self.stuff.machine_cls(states=states, transitions=transitions, initial='A', auto_transitions=False)
+        self.assertEqual(10, len(machine.get_transitions()))
+        self.assertEqual(2, len(machine.get_transitions(source='A')))
+        self.assertEqual(2, len(machine.get_transitions('inner')))
+        self.assertEqual(3, len(machine.get_transitions('mid')))
+        self.assertEqual(3, len(machine.get_transitions(dest='B{0}1'.format(seperator))))
+        self.assertEqual(2, len(machine.get_transitions(source='B{0}2'.format(seperator),
+                                                        dest='B{0}3'.format(seperator))))
+        self.assertEqual(1, len(machine.get_transitions(source='B{0}3{0}a'.format(seperator),
+                                                        dest='B{0}3{0}b'.format(seperator))))
+        self.assertEqual(1, len(machine.get_transitions(source=machine.states['B'].states['3'].states['b'])))
+        # should be B_3_b -> B_3_a, B_3 -> B_1 and B -> C
+        self.assertEqual(3, len(machine.get_transitions(source=machine.states['B'].states['3'].states['b'],
+                                                        delegate=True)))
+
     def test_internal_transitions(self):
         s = self.stuff
         s.machine.add_transition('internal', 'A', None, prepare='increase_level')
@@ -520,32 +459,293 @@ class TestTransitions(TestsCore):
         self.assertEqual(s.state, 'A')
         self.assertEqual(s.level, 2)
 
+    def test_transition_with_unknown_state(self):
+        s = self.stuff
+        with self.assertRaises(ValueError):
+            s.machine.add_transition('next', 'A', s.machine.state_cls('X'))
 
-@skipIf(pgv is None, 'NestedGraph diagram test requires graphviz')
-class TestWithGraphTransitions(TestTransitions):
+    def test_skip_to_override(self):
+        mock = MagicMock()
+
+        class Model:
+
+            def to(self):
+                mock()
+
+        model1 = Model()
+        model2 = DummyModel()
+        machine = self.machine_cls([model1, model2], states=['A', 'B'], initial='A')
+        model1.to()
+        model2.to('B')
+        self.assertTrue(mock.called)
+        self.assertTrue(model2.is_B())
+
+    def test_trigger_parent(self):
+        parent_mock = MagicMock()
+        exit_mock = MagicMock()
+        enter_mock = MagicMock()
+
+        class Model:
+
+            def on_exit_A(self):
+                parent_mock()
+
+            def on_exit_A_1(self):
+                exit_mock()
+
+            def on_enter_A_2(self):
+                enter_mock()
+
+        model = Model()
+        machine = self.machine_cls(model, states=[{'name': 'A', 'children': ['1', '2']}],
+                                   transitions=[['go', 'A', 'A_2'], ['enter', 'A', 'A_1']], initial='A')
+
+        model.enter()
+        self.assertFalse(parent_mock.called)
+        model.go()
+        self.assertTrue(exit_mock.called)
+        self.assertTrue(enter_mock.called)
+        self.assertFalse(parent_mock.called)
+
+    def test_trigger_parent_model_self(self):
+        parent_mock = MagicMock()
+        exit_mock = MagicMock()
+        enter_mock = MagicMock()
+
+        class CustomMachine(self.machine_cls):
+
+            def on_exit_A(self):
+                parent_mock()
+
+            def on_exit_A_1(self):
+                exit_mock()
+
+            def on_enter_A_2(self):
+                enter_mock()
+
+        machine = CustomMachine(states=[{'name': 'A', 'children': ['1', '2']}],
+                                transitions=[['go', 'A', 'A_2'], ['enter', 'A', 'A_1']], initial='A')
+        machine.enter()
+        self.assertFalse(parent_mock.called)
+        machine.go()
+        self.assertTrue(exit_mock.called)
+        self.assertTrue(enter_mock.called)
+        self.assertFalse(parent_mock.called)
+
+    def test_child_condition_persistence(self):
+        # even though the transition is invalid for the parent it is valid for the nested child state
+        # no invalid transition exception should be thrown
+        machine = self.machine_cls(states=[{'name': 'A', 'children': ['1', '2'], 'initial': '1',
+                                            'transitions': [{'trigger': 'go', 'source': '1', 'dest': '2',
+                                                             'conditions': self.stuff.this_fails}]}, 'B'],
+                                   transitions=[['go', 'B', 'A']], initial='A')
+        self.assertFalse(False, machine.go())
+
+    def test_exception_in_state_enter_exit(self):
+        # https://github.com/pytransitions/transitions/issues/486
+        # NestedState._scope needs to be reset when an error is raised in a state callback
+        class Model:
+            def on_enter_B_1(self):
+                raise RuntimeError("Oh no!")
+
+            def on_exit_C_1(self):
+                raise RuntimeError("Oh no!")
+
+        states = ['A',
+                  {'name': 'B', 'initial': '1', 'children': ['1', '2']},
+                  {'name': 'C', 'initial': '1', 'children': ['1', '2']}]
+        model = Model()
+        machine = self.machine_cls(model, states=states, initial='A')
+        with self.assertRaises(RuntimeError):
+            model.to_B()
+        self.assertTrue(model.is_B_1())
+        machine.set_state('A', model)
+        with self.assertRaises(RuntimeError):
+            model.to_B()
+        with self.assertRaises(RuntimeError):
+            model.to_C()
+            model.to_A()
+        self.assertTrue(model.is_C_1())
+        machine.set_state('A', model)
+        model.to_C()
+        self.assertTrue(model.is_C_1())
+
+
+class TestSeparatorsBase(TestCase):
+
+    separator = default_separator
 
     def setUp(self):
-        State.separator = state_separator
-        states = ['A', 'B', {'name': 'C', 'children': ['1', '2', {'name': '3', 'children': ['a', 'b', 'c']}]},
-                  'D', 'E', 'F']
 
-        machine_cls = MachineFactory.get_predefined(graph=True, nested=True)
-        self.stuff = Stuff(states, machine_cls)
+        class CustomNestedState(NestedState):
+            separator = self.separator
 
+        class CustomHierarchicalMachine(MachineFactory.get_predefined(nested=True)):
+            state_cls = CustomNestedState
+
+        self.states = test_states
+        self.machine_cls = CustomHierarchicalMachine
+        self.state_cls = CustomNestedState
+        self.stuff = Stuff(self.states, self.machine_cls)
+
+    def test_enter_exit_nested(self):
+        separator = self.state_cls.separator
+        s = self.stuff
+        s.machine.add_transition('advance', 'A', 'C{0}3'.format(separator))
+        s.machine.add_transition('reverse', 'C', 'A')
+        s.machine.add_transition('lower', ['C{0}1'.format(separator),
+                                           'C{0}3'.format(separator)], 'C{0}3{0}a'.format(separator))
+        s.machine.add_transition('rise', 'C{0}3'.format(separator), 'C{0}1'.format(separator))
+        s.machine.add_transition('fast', 'A', 'C{0}3{0}a'.format(separator))
+
+        for state_name in s.machine.get_nested_state_names():
+            state = s.machine.get_state(state_name)
+            state.on_enter.append('increase_level')
+            state.on_exit.append('decrease_level')
+
+        s.advance()
+        self.assertEqual('C{0}3'.format(separator), s.state)
+        self.assertEqual(2, s.level)
+        self.assertEqual(3, s.transitions)  # exit A; enter C,3
+        s.lower()
+        self.assertEqual(s.state, 'C{0}3{0}a'.format(separator))
+        self.assertEqual(3, s.level)
+        self.assertEqual(4, s.transitions)  # enter a
+        s.rise()
+        self.assertEqual('C%s1' % separator, s.state)
+        self.assertEqual(2, s.level)
+        self.assertEqual(7, s.transitions)  # exit a, 3; enter 1
+        s.reverse()
+        self.assertEqual('A', s.state)
+        self.assertEqual(1, s.level)
+        self.assertEqual(10, s.transitions)  # exit 1, C; enter A
+        s.fast()
+        self.assertEqual(s.state, 'C{0}3{0}a'.format(separator))
+        self.assertEqual(s.level, 3)
+        self.assertEqual(s.transitions, 14)  # exit A; enter C, 3, a
+        s.to_A()
+        self.assertEqual(s.state, 'A')
+        self.assertEqual(s.level, 1)
+        self.assertEqual(s.transitions, 18)  # exit a, 3, C; enter A
+        s.to_A()
+        self.assertEqual(s.state, 'A')
+        self.assertEqual(s.level, 1)
+        self.assertEqual(s.transitions, 20)  # exit A; enter A
+        if separator == '_':
+            s.to_C_3_a()
+        else:
+            print("separator", separator)
+            s.to_C.s3.a()
+        self.assertEqual(s.state, 'C{0}3{0}a'.format(separator))
+        self.assertEqual(s.level, 3)
+        self.assertEqual(s.transitions, 24)  # exit A; enter C, 3, a
+
+    def test_state_change_listeners(self):
+        State = self.state_cls
+        s = self.stuff
+        s.machine.add_transition('advance', 'A', 'C%s1' % State.separator)
+        s.machine.add_transition('reverse', 'C', 'A')
+        s.machine.add_transition('lower', 'C%s1' % State.separator, 'C{0}3{0}a'.format(State.separator))
+        s.machine.add_transition('rise', 'C%s3' % State.separator, 'C%s1' % State.separator)
+        s.machine.add_transition('fast', 'A', 'C{0}3{0}a'.format(State.separator))
+        s.machine.on_enter_C('hello_world')
+        s.machine.on_exit_C('goodbye')
+        s.machine.on_enter('C{0}3{0}a'.format(State.separator), 'greet')
+        s.machine.on_exit('C%s3' % State.separator, 'meet')
+        s.advance()
+        self.assertEqual(s.state, 'C%s1' % State.separator)
+        self.assertEqual(s.message, 'Hello World!')
+        s.lower()
+        self.assertEqual(s.state, 'C{0}3{0}a'.format(State.separator))
+        self.assertEqual(s.message, 'Hi')
+        s.rise()
+        self.assertEqual(s.state, 'C%s1' % State.separator)
+        self.assertTrue(s.message.startswith('Nice to'))
+        s.reverse()
+        self.assertEqual(s.state, 'A')
+        self.assertTrue(s.message.startswith('So long'))
+        s.fast()
+        self.assertEqual(s.state, 'C{0}3{0}a'.format(State.separator))
+        self.assertEqual(s.message, 'Hi')
+        s.to_A()
+        self.assertEqual(s.state, 'A')
+        self.assertTrue(s.message.startswith('So long'))
+
+    def test_nested_auto_transitions(self):
+        State = self.state_cls
+        s = self.stuff
+        s.to_C()
+        self.assertEqual(s.state, 'C')
+        state = 'C{0}3{0}a'.format(State.separator)
+        s.to(state)
+        self.assertEqual(s.state, state)
+        # backwards compatibility check (can be removed in 0.7)
+        self.assertEqual(s.state, state)
+        for state_name in s.machine.get_nested_state_names():
+            event_name = 'to_{0}'.format(state_name)
+            num_base_states = len(s.machine.states)
+            self.assertTrue(event_name in s.machine.events)
+            self.assertEqual(len(s.machine.events[event_name].transitions), num_base_states)
+
+    @skipIf(pgv is None, 'NestedGraph diagram test requires graphviz')
     def test_ordered_with_graph(self):
         GraphMachine = MachineFactory.get_predefined(graph=True, nested=True)
 
+        class CustomHierarchicalGraphMachine(GraphMachine):
+            state_cls = self.state_cls
+
         states = ['A', 'B', {'name': 'C', 'children': ['1', '2',
                                                        {'name': '3', 'children': ['a', 'b', 'c']}]}, 'D', 'E', 'F']
 
-        State.separator = '/'
-        machine = GraphMachine('self', states, initial='A',
-                               auto_transitions=False,
-                               ignore_invalid_triggers=True)
+        machine = CustomHierarchicalGraphMachine('self', states, initial='A', auto_transitions=False,
+                                                 ignore_invalid_triggers=True, use_pygraphviz=False)
         machine.add_ordered_transitions(trigger='next_state')
         machine.next_state()
         self.assertEqual(machine.state, 'B')
-        target = tempfile.NamedTemporaryFile()
+        target = tempfile.NamedTemporaryFile(suffix='.png', delete=False)
         machine.get_graph().draw(target.name, prog='dot')
         self.assertTrue(getsize(target.name) > 0)
         target.close()
+        unlink(target.name)
+
+    def test_example_two(self):
+        separator = self.state_cls.separator
+        states = ['A', 'B',
+                  {'name': 'C', 'children': ['1', '2',
+                                             {'name': '3', 'children': ['a', 'b', 'c']}]
+                   }]
+
+        transitions = [
+            ['reset', 'C', 'A'],
+            ['reset', 'C%s2' % separator, 'C']  # overwriting parent reset
+        ]
+
+        # we rely on auto transitions
+        machine = self.stuff.machine_cls(states=states, transitions=transitions, initial='A')
+
+        machine.to_B()  # exit state A, enter state B
+        machine.to_C()  # exit B, enter C
+        if separator == '_':
+            machine.to_C_3_a()
+        else:
+            machine.to_C.s3.a()  # enter C↦a; enter C↦3↦a;
+        self.assertEqual(machine.state, 'C{0}3{0}a'.format(separator))
+        machine.to('C{0}2'.format(separator))  # exit C↦3↦a, exit C↦3, enter C↦2
+        self.assertEqual(machine.state, 'C{0}2'.format(separator))
+        machine.reset()  # exit C↦2; reset C has been overwritten by C↦3
+        self.assertEqual('C', machine.state)
+        machine.reset()  # exit C, enter A
+        self.assertEqual('A', machine.state)
+
+
+class TestSeparatorsSlash(TestSeparatorsBase):
+    separator = '/'
+
+
+class TestSeparatorsDot(TestSeparatorsBase):
+    separator = '.'
+
+
+@skipIf(sys.version_info[0] < 3, "Unicode separators are only supported for Python 3")
+class TestSeparatorUnicode(TestSeparatorsBase):
+    separator = u'↦'
