@@ -31,10 +31,13 @@ import logging
 from collections import OrderedDict, defaultdict, deque
 from functools import partial
 from six import string_types
+import warnings
 
 _LOGGER = logging.getLogger(__name__)
 _LOGGER.addHandler(logging.NullHandler())
 
+warnings.filterwarnings(action='default', message=r".*transitions version.*", category=DeprecationWarning)
+
 
 def listify(obj):
     """Wraps a passed object into a list in case it has not been a list, tuple before.
@@ -50,24 +53,6 @@ def listify(obj):
     return obj if isinstance(obj, (list, tuple, EnumMeta)) else [obj]
 
 
-def _get_trigger(model, machine, trigger_name, *args, **kwargs):
-    """Convenience function added to the model to trigger events by name.
-    Args:
-        model (object): Model with assigned event trigger.
-        machine (Machine): The machine containing the evaluated events.
-        trigger_name (str): Name of the trigger to be called.
-        *args: Variable length argument list which is passed to the triggered event.
-        **kwargs: Arbitrary keyword arguments which is passed to the triggered event.
-    Returns:
-        bool: True if a transitions has been conducted or the trigger event has been queued.
-    """
-    try:
-        event = machine.events[trigger_name]
-    except KeyError:
-        raise AttributeError("Do not know event named '%s'." % trigger_name)
-    return event.trigger(model, *args, **kwargs)
-
-
 def _prep_ordered_arg(desired_length, arguments=None):
     """Ensure list of arguments passed to add_ordered_transitions has the proper length.
     Expands the given arguments and apply same condition, callback
@@ -106,7 +91,7 @@ class State(object):
                  ignore_invalid_triggers=None):
         """
         Args:
-            name (str): The name of the state
+            name (str or Enum): The name of the state
             on_enter (str or list): Optional callable(s) to trigger when a
                 state is entered. Can be either a string providing the name of
                 a callable, or a list of strings.
@@ -136,16 +121,14 @@ class State(object):
     def enter(self, event_data):
         """ Triggered when a state is entered. """
         _LOGGER.debug("%sEntering state %s. Processing callbacks...", event_data.machine.name, self.name)
-        for handle in self.on_enter:
-            event_data.machine.callback(handle, event_data)
-        _LOGGER.info("%sEntered state %s", event_data.machine.name, self.name)
+        event_data.machine.callbacks(self.on_enter, event_data)
+        _LOGGER.info("%sFinished processing state %s enter callbacks.", event_data.machine.name, self.name)
 
     def exit(self, event_data):
         """ Triggered when a state is exited. """
         _LOGGER.debug("%sExiting state %s. Processing callbacks...", event_data.machine.name, self.name)
-        for handle in self.on_exit:
-            event_data.machine.callback(handle, event_data)
-        _LOGGER.info("%sExited state %s", event_data.machine.name, self.name)
+        event_data.machine.callbacks(self.on_exit, event_data)
+        _LOGGER.info("%sFinished processing state %s exit callbacks.", event_data.machine.name, self.name)
 
     def add_callback(self, trigger, func):
         """ Add a new enter or exit callback.
@@ -220,8 +203,11 @@ class Transition(object):
             but only if condition checks have been successful.
     """
 
-    # A list of dynamic methods which can be resolved by a ``Machine`` instance for convenience functions.
     dynamic_methods = ['before', 'after', 'prepare']
+    """ A list of dynamic methods which can be resolved by a ``Machine`` instance for convenience functions. """
+    condition_cls = Condition
+    """ The class used to wrap condition checks. Can be replaced to alter condition resolution behaviour
+        (e.g. OR instead of AND for 'conditions' or AND instead of OR for 'unless') """
 
     def __init__(self, source, dest, conditions=None, unless=None, before=None,
                  after=None, prepare=None):
@@ -250,10 +236,18 @@ class Transition(object):
         self.conditions = []
         if conditions is not None:
             for cond in listify(conditions):
-                self.conditions.append(Condition(cond))
+                self.conditions.append(self.condition_cls(cond))
         if unless is not None:
             for cond in listify(unless):
-                self.conditions.append(Condition(cond, target=False))
+                self.conditions.append(self.condition_cls(cond, target=False))
+
+    def _eval_conditions(self, event_data):
+        for cond in self.conditions:
+            if not cond.check(event_data):
+                _LOGGER.debug("%sTransition condition failed: %s() does not return %s. Transition halted.",
+                              event_data.machine.name, cond.func, cond.target)
+                return False
+        return True
 
     def execute(self, event_data):
         """ Execute the transition.
@@ -264,27 +258,21 @@ class Transition(object):
         """
         _LOGGER.debug("%sInitiating transition from state %s to state %s...",
                       event_data.machine.name, self.source, self.dest)
-        machine = event_data.machine
 
-        for func in self.prepare:
-            machine.callback(func, event_data)
-            _LOGGER.debug("Executed callback '%s' before conditions.", func)
+        event_data.machine.callbacks(self.prepare, event_data)
+        _LOGGER.debug("%sExecuted callbacks before conditions.", event_data.machine.name)
 
-        for cond in self.conditions:
-            if not cond.check(event_data):
-                _LOGGER.debug("%sTransition condition failed: %s() does not return %s. Transition halted.",
-                              event_data.machine.name, cond.func, cond.target)
-                return False
-        for func in itertools.chain(machine.before_state_change, self.before):
-            machine.callback(func, event_data)
-            _LOGGER.debug("%sExecuted callback '%s' before transition.", event_data.machine.name, func)
+        if not self._eval_conditions(event_data):
+            return False
+
+        event_data.machine.callbacks(itertools.chain(event_data.machine.before_state_change, self.before), event_data)
+        _LOGGER.debug("%sExecuted callback before transition.", event_data.machine.name)
 
         if self.dest:  # if self.dest is None this is an internal transition with no actual state change
             self._change_state(event_data)
 
-        for func in itertools.chain(self.after, machine.after_state_change):
-            machine.callback(func, event_data)
-            _LOGGER.debug("%sExecuted callback '%s' after transition.", event_data.machine.name, func)
+        event_data.machine.callbacks(itertools.chain(self.after, event_data.machine.after_state_change), event_data)
+        _LOGGER.debug("%sExecuted callback after transition.", event_data.machine.name)
         return True
 
     def _change_state(self, event_data):
@@ -423,9 +411,8 @@ class Event(object):
         return self._process(event_data)
 
     def _process(self, event_data):
-        for func in self.machine.prepare_event:
-            self.machine.callback(func, event_data)
-            _LOGGER.debug("Executed machine preparation callback '%s' before conditions.", func)
+        self.machine.callbacks(self.machine.prepare_event, event_data)
+        _LOGGER.debug("%sExecuted machine preparation callbacks before conditions.", self.machine.name)
 
         try:
             for trans in self.transitions[event_data.state.name]:
@@ -437,9 +424,9 @@ class Event(object):
             event_data.error = err
             raise
         finally:
-            for func in self.machine.finalize_event:
-                self.machine.callback(func, event_data)
-                _LOGGER.debug("Executed machine finalize callback '%s'.", func)
+            self.machine.callbacks(self.machine.finalize_event, event_data)
+            _LOGGER.debug("%sExecuted machine finalize callbacks", self.machine.name)
+
         return event_data.result
 
     def __repr__(self):
@@ -602,12 +589,12 @@ class Machine(object):
         for mod in models:
             mod = self if mod == 'self' else mod
             if mod not in self.models:
-                self._checked_assignment(mod, 'trigger', partial(_get_trigger, mod, self))
+                self._checked_assignment(mod, 'trigger', partial(self._get_trigger, mod))
 
-                for trigger, _ in self.events.items():
+                for trigger in self.events:
                     self._add_trigger_to_model(trigger, mod)
 
-                for _, state in self.states.items():
+                for state in self.states.values():
                     self._add_model_to_state(state, mod)
 
                 self.set_state(initial, model=mod)
@@ -615,11 +602,16 @@ class Machine(object):
 
     def remove_model(self, model):
         """ Remove a model from the state machine. The model will still contain all previously added triggers
-        and callbacks, but will not receive updates when states or transitions are added to the Machine. """
+        and callbacks, but will not receive updates when states or transitions are added to the Machine.
+        If an event queue is used, all queued events of that model will be removed."""
         models = listify(model)
 
         for mod in models:
             self.models.remove(mod)
+        if len(self._transition_queue) > 0:
+            # the first element of the list is currently executed. Keeping it for further Machine._process(ing)
+            self._transition_queue = deque(
+                [self._transition_queue[0]] + [e for e in self._transition_queue if e.args[0] not in models])
 
     @classmethod
     def _create_transition(cls, *args, **kwargs):
@@ -644,7 +636,7 @@ class Machine(object):
             if value.name not in self.states:
                 self.add_state(value)
             else:
-                assert self._has_state(value)
+                _ = self._has_state(value, raise_error=True)
             self._initial = value.name
         else:
             state_name = value.name if isinstance(value, Enum) else value
@@ -738,9 +730,11 @@ class Machine(object):
         """
             Set the current state.
         Args:
-            state (str or Enum): value of setted state
+            state (str or Enum or State): value of state to be set
+            model (optional[object]): targeted model; if not set, all models will be set to 'state'
         """
-        state = self.get_state(state)
+        if not isinstance(state, State):
+            state = self.get_state(state)
         models = self.models if model is None else listify(model)
 
         for mod in models:
@@ -791,13 +785,40 @@ class Machine(object):
             self.states[state.name] = state
             for model in self.models:
                 self._add_model_to_state(state, model)
-        # Add automatic transitions after all states have been created
-        if self.auto_transitions:
-            for state in self.states.keys():
-                self.add_transition('to_%s' % state, self.wildcard_all, state)
+            if self.auto_transitions:
+                for a_state in self.states.keys():
+                    # add all states as sources to auto transitions 'to_<state>' with dest <state>
+                    if a_state == state.name:
+                        if self.model_attribute == 'state':
+                            method_name = 'to_%s' % a_state
+                        else:
+                            method_name = 'to_%s_%s' % (self.model_attribute, a_state)
+                            self.add_transition('to_%s' % a_state, self.wildcard_all, a_state,
+                                                prepare=partial(_warning_wrapper_to, 'to_%s' % a_state))
+                        self.add_transition(method_name, self.wildcard_all, a_state)
+
+                    # add auto transition with source <state> to <a_state>
+                    else:
+                        if self.model_attribute == 'state':
+                            method_name = 'to_%s' % a_state
+                        else:
+                            method_name = 'to_%s_%s' % (self.model_attribute, a_state)
+                            self.add_transition('to_%s' % a_state, state.name, a_state,
+                                                prepare=partial(_warning_wrapper_to, 'to_%s' % a_state))
+                        self.add_transition(method_name, state.name, a_state)
 
     def _add_model_to_state(self, state, model):
-        self._checked_assignment(model, 'is_%s' % state.name, partial(self.is_state, state.value, model))
+        # Add convenience function 'is_<state_name>' (e.g. 'is_A') to the model.
+        # When model_attribute has been customized, add 'is_<model_attribute>_<state_name>' instead
+        # to potentially support multiple states on one model (e.g. 'is_custom_state_A' and 'is_my_state_B').
+
+        func = partial(self.is_state, state.value, model)
+        if self.model_attribute == 'state':
+            method_name = 'is_%s' % state.name
+        else:
+            method_name = 'is_%s_%s' % (self.model_attribute, state.name)
+            self._checked_assignment(model, 'is_%s' % state.name, partial(_warning_wrapper_is, method_name, func))
+        self._checked_assignment(model, method_name, func)
 
         # Add dynamic method callbacks (enter/exit) if there are existing bound methods in the model
         # except if they are already mentioned in 'on_enter/exit' of the defined state
@@ -816,6 +837,27 @@ class Machine(object):
     def _add_trigger_to_model(self, trigger, model):
         self._checked_assignment(model, trigger, partial(self.events[trigger].trigger, model))
 
+    def _get_trigger(self, model, trigger_name, *args, **kwargs):
+        """Convenience function added to the model to trigger events by name.
+        Args:
+            model (object): Model with assigned event trigger.
+            trigger_name (str): Name of the trigger to be called.
+            *args: Variable length argument list which is passed to the triggered event.
+            **kwargs: Arbitrary keyword arguments which is passed to the triggered event.
+        Returns:
+            bool: True if a transitions has been conducted or the trigger event has been queued.
+        """
+        try:
+            event = self.events[trigger_name]
+        except KeyError:
+            state = self.get_model_state(model)
+            ignore = state.ignore_invalid_triggers if state.ignore_invalid_triggers is not None \
+                else self.ignore_invalid_triggers
+            if not ignore:
+                raise AttributeError("Do not know event named '%s'." % trigger_name)
+            return False
+        return event.trigger(model, *args, **kwargs)
+
     def get_triggers(self, *args):
         """ Collects all triggers FROM certain states.
         Args:
@@ -867,12 +909,20 @@ class Machine(object):
         if source == self.wildcard_all:
             source = list(self.states.keys())
         else:
-            source = [s.name if self._has_state(s) or isinstance(s, Enum) else s for s in listify(source)]
+            # states are checked lazily which means we will only raise exceptions when the passed state
+            # is a State object because of potential confusion (see issue #155 for more details)
+            source = [s.name if isinstance(s, State) and self._has_state(s, raise_error=True) or hasattr(s, 'name') else
+                      s for s in listify(source)]
 
         for state in source:
-            _dest = state if dest == self.wildcard_same else dest
-            if _dest and self._has_state(_dest) or isinstance(_dest, Enum):
-                _dest = _dest.name
+            if dest == self.wildcard_same:
+                _dest = state
+            elif dest is not None:
+                if isinstance(dest, State):
+                    _ = self._has_state(dest, raise_error=True)
+                _dest = dest.name if hasattr(dest, 'name') else dest
+            else:
+                _dest = None
             _trans = self._create_transition(state, _dest, conditions, unless, before,
                                              after, prepare, **kwargs)
             self.events[trigger].add_transition(_trans)
@@ -906,7 +956,9 @@ class Machine(object):
                 state to the first state.
             loop_includes_initial (boolean): If no initial state was defined in
                 the machine, setting this to True will cause the _initial state
-                placeholder to be included in the added transitions.
+                placeholder to be included in the added transitions. This argument
+                has no effect if the states argument is passed without the
+                initial state included.
             conditions (str or list): Condition(s) that must pass in order
                 for the transition to take place. Either a list providing the
                 name of a callable, or a list of callables. For the transition
@@ -935,8 +987,13 @@ class Machine(object):
         after = _prep_ordered_arg(len_transitions, after)
         prepare = _prep_ordered_arg(len_transitions, prepare)
         # reorder list so that the initial state is actually the first one
-        idx = states.index(self._initial)
-        states = states[idx:] + states[:idx]
+        try:
+            idx = states.index(self._initial)
+            states = states[idx:] + states[:idx]
+            first_in_loop = states[0 if loop_includes_initial else 1]
+        except ValueError:
+            # since initial is not part of states it shouldn't be part of the loop either
+            first_in_loop = states[0]
 
         for i in range(0, len(states) - 1):
             self.add_transition(trigger, states[i], states[i + 1],
@@ -949,7 +1006,7 @@ class Machine(object):
         if loop:
             self.add_transition(trigger, states[-1],
                                 # omit initial if not loop_includes_initial
-                                states[0 if loop_includes_initial else 1],
+                                first_in_loop,
                                 conditions=conditions[-1],
                                 unless=unless[-1],
                                 before=before[-1],
@@ -961,22 +1018,26 @@ class Machine(object):
         """ Return the transitions from the Machine.
         Args:
             trigger (str): Trigger name of the transition.
-            source (str): Limits removal to transitions from a certain state.
-            dest (str): Limits removal to transitions to a certain state.
+            source (str, Enum or State): Limits list to transitions from a certain state.
+            dest (str, Enum or State): Limits list to transitions to a certain state.
         """
         if trigger:
-            events = (self.events[trigger], )
+            try:
+                events = (self.events[trigger], )
+            except KeyError:
+                return []
         else:
             events = self.events.values()
         transitions = []
         for event in events:
             transitions.extend(
                 itertools.chain.from_iterable(event.transitions.values()))
+        target_source = source.name if hasattr(source, 'name') else source if source != "*" else ""
+        target_dest = dest.name if hasattr(dest, 'name') else dest if dest != "*" else ""
         return [transition
                 for transition in transitions
-                if (transition.source, transition.dest) == (
-                    source if source != "*" else transition.source,
-                    dest if dest != "*" else transition.dest)]
+                if (transition.source, transition.dest) == (target_source or transition.source,
+                                                            target_dest or transition.dest)]
 
     def remove_transition(self, trigger, source="*", dest="*"):
         """ Removes a transition from the Machine and all models.
@@ -1016,6 +1077,12 @@ class Machine(object):
         """
         return all([getattr(model, trigger)(*args, **kwargs) for model in self.models])
 
+    def callbacks(self, funcs, event_data):
+        """ Triggers a list of callbacks """
+        for func in funcs:
+            self.callback(func, event_data)
+            _LOGGER.info("%sExecuted callback '%s'", self.name, func)
+
     def callback(self, func, event_data):
         """ Trigger a callback function with passed event_data parameters. In case func is a string,
             the callable will be resolved from the passed model in event_data. This function is not intended to
@@ -1038,10 +1105,10 @@ class Machine(object):
 
     @staticmethod
     def resolve_callable(func, event_data):
-        """ Converts a model's method name or a path to a callable into a callable.
+        """ Converts a model's property name, method name or a path to a callable into a callable.
             If func is not a string it will be returned unaltered.
         Args:
-            func (str or callable): Method name or a path to a callable
+            func (str or callable): Property name, method name or a path to a callable
             event_data (EventData): Currently processed event
         Returns:
             callable function resolved from string or func
@@ -1049,6 +1116,10 @@ class Machine(object):
         if isinstance(func, string_types):
             try:
                 func = getattr(event_data.model, func)
+                if not callable(func):  # if a property or some other not callable attribute was passed
+                    def func_wrapper(*_, **__):  # properties cannot process parameters
+                        return func
+                    return func_wrapper
             except AttributeError:
                 try:
                     mod, name = func.rsplit('.', 1)
@@ -1061,14 +1132,12 @@ class Machine(object):
                                          "model nor imported from a module." % func)
         return func
 
-    def _has_state(self, state):
-        if isinstance(state, State):
-            if state in self.states.values():
-                return True
-            else:
-                raise ValueError('State %s has not been added to the machine' % state.name)
-        else:
-            return False
+    def _has_state(self, state, raise_error=False):
+        found = state in self.states.values()
+        if not found and raise_error:
+            msg = 'State %s has not been added to the machine' % (state.name if hasattr(state, 'name') else state)
+            raise ValueError(msg)
+        return found
 
     def _process(self, trigger):
 
@@ -1137,8 +1206,11 @@ class Machine(object):
                 state = self.get_state(target)
                 return partial(state.add_callback, callback_type[3:])
 
-        # Nothing matched
-        raise AttributeError("'{}' does not exist on <Machine@{}>".format(name, id(self)))
+        try:
+            return self.__getattribute__(name)
+        except AttributeError:
+            # Nothing matched
+            raise AttributeError("'{}' does not exist on <Machine@{}>".format(name, id(self)))
 
 
 class MachineError(Exception):
@@ -1152,3 +1224,21 @@ class MachineError(Exception):
 
     def __str__(self):
         return repr(self.value)
+
+
+# TODO: Remove in 0.9.0
+def _warning_wrapper_is(meth_name, func, *args, **kwargs):
+    warnings.warn("Starting from transitions version 0.8.3, 'is_<state_name>' convenience functions will be"
+                  " assigned to 'is_<model_attribute>_<state_name>' when 'model_attribute "
+                  "!= \"state\"'. In 0.9.0, 'is_<state_name>' will NOT be assigned anymore when "
+                  "'model_attribute != \"state\"'! Please adjust your code and use "
+                  "'{0}' instead.".format(meth_name), DeprecationWarning)
+    return func(*args, **kwargs)
+
+
+def _warning_wrapper_to(meth_name, *args, **kwargs):
+    warnings.warn("Starting from transitions version 0.8.3, 'to_<state_name>' convenience functions will be"
+                  " assigned to 'to_<model_attribute>_<state_name>' when 'model_attribute "
+                  "!= \"state\"'. In 0.9.0, 'to_<state_name>' will NOT be assigned anymore when "
+                  "'model_attribute != \"state\"'! Please adjust your code and use "
+                  "'{0}' instead.".format(meth_name), DeprecationWarning)
