@@ -3,7 +3,7 @@ import logging
 import asyncio
 import contextvars
 import inspect
-
+from collections import defaultdict, deque
 from functools import partial, reduce
 import copy
 
@@ -30,13 +30,13 @@ class AsyncState(State):
         """ Triggered when a state is entered. """
         _LOGGER.debug("%sEntering state %s. Processing callbacks...", event_data.machine.name, self.name)
         await event_data.machine.callbacks(self.on_enter, event_data)
-        _LOGGER.info("%sEntered state %s", event_data.machine.name, self.name)
+        _LOGGER.info("%sFinished processing state %s enter callbacks.", event_data.machine.name, self.name)
 
     async def exit(self, event_data):
         """ Triggered when a state is exited. """
         _LOGGER.debug("%sExiting state %s. Processing callbacks...", event_data.machine.name, self.name)
         await event_data.machine.callbacks(self.on_exit, event_data)
-        _LOGGER.info("%sExited state %s", event_data.machine.name, self.name)
+        _LOGGER.info("%sFinished processing state %s exit callbacks.", event_data.machine.name, self.name)
 
 
 class NestedAsyncState(NestedState, AsyncState):
@@ -120,15 +120,17 @@ class AsyncTransition(Transition):
 
         machine = event_data.machine
         # cancel running tasks since the transition will happen
-        machine.switch_model_context(event_data.model)
+        await machine.switch_model_context(event_data.model)
 
-        await event_data.machine.callbacks(itertools.chain(event_data.machine.before_state_change, self.before), event_data)
+        await event_data.machine.callbacks(event_data.machine.before_state_change, event_data)
+        await event_data.machine.callbacks(self.before, event_data)
         _LOGGER.debug("%sExecuted callback before transition.", event_data.machine.name)
 
         if self.dest:  # if self.dest is None this is an internal transition with no actual state change
             await self._change_state(event_data)
 
-        await event_data.machine.callbacks(itertools.chain(self.after, event_data.machine.after_state_change), event_data)
+        await event_data.machine.callbacks(self.after, event_data)
+        await event_data.machine.callbacks(event_data.machine.after_state_change, event_data)
         _LOGGER.debug("%sExecuted callback after transition.", event_data.machine.name)
         return True
 
@@ -301,8 +303,23 @@ class AsyncMachine(Machine):
     transition_cls = AsyncTransition
     event_cls = AsyncEvent
     async_tasks = {}
+    protected_tasks = []
     current_context = contextvars.ContextVar('current_context', default=None)
 
+    def __init__(self, *args, **kwargs):
+        self._transition_queue_dict = {}
+        super().__init__(*args, **kwargs)
+        if self.has_queue is True:
+            self._transition_queue_dict = _DictionaryMock()
+            # _DictionaryMock sets and returns ONE internal value and ignores the passed key
+            self._transition_queue_dict[0] = self._transition_queue
+
+    def add_model(self, model, initial=None):
+        super().add_model(model, initial)
+        if self.has_queue == 'model':
+            for mod in listify(model):
+                self._transition_queue_dict[mod if mod != 'self' else self] = deque()
+
     async def dispatch(self, trigger, *args, **kwargs):  # ToDo: not tested
         """ Trigger an event on all models assigned to the machine.
         Args:
@@ -349,19 +366,19 @@ class AsyncMachine(Machine):
         """
         return await asyncio.gather(*[func() for func in callables])
 
-    def switch_model_context(self, model):
+    async def switch_model_context(self, model):
         """
         This method is called by an `AsyncTransition` when all conditional tests have passed and the transition will happen.
         This requires already running tasks to be cancelled.
         Args:
             model (object): The currently processed model
         """
-        running_task = self.async_tasks.get(model, None)
-        if self.current_context.get() != running_task:
-            if running_task is not None and running_task.done() is False:
+        for running_task in self.async_tasks.get(model, []):
+            if self.current_context.get() == running_task or running_task in self.protected_tasks:
+                continue
+            elif running_task.done() is False:
                 _LOGGER.debug("Cancel running tasks...")
                 running_task.cancel()
-            self.async_tasks[model] = self.current_context.get()
 
     async def process_context(self, func, model):
         """
@@ -376,13 +393,37 @@ class AsyncMachine(Machine):
         """
         if self.current_context.get() is None:
             self.current_context.set(asyncio.current_task())
+            if model in self.async_tasks:
+                self.async_tasks[model].append(asyncio.current_task())
+            else:
+                self.async_tasks[model] = [asyncio.current_task()]
             try:
-                return await self._process(func)
+                res = await self._process(func, model)
             except asyncio.CancelledError:
-                return False
-        return await self._process(func)
+                res = False
+            finally:
+                self.async_tasks[model].remove(asyncio.current_task())
+                if len(self.async_tasks[model]) == 0:
+                    del self.async_tasks[model]
+        else:
+            res = await self._process(func, model)
+        return res
 
-    async def _process(self, trigger):
+    def remove_model(self, model):
+        """ Remove a model from the state machine. The model will still contain all previously added triggers
+        and callbacks, but will not receive updates when states or transitions are added to the Machine.
+        If an event queue is used, all queued events of that model will be removed."""
+        models = listify(model)
+        if self.has_queue == 'model':
+            for mod in models:
+                del self._transition_queue_dict[mod]
+        if len(self._transition_queue) > 0:
+            queue = self._transition_queue
+            new_queue = [queue.popleft()] + [e for e in queue if e.args[0] not in models]
+            self._transition_queue.clear()
+            self._transition_queue.extend(new_queue)
+
+    async def _process(self, trigger, model):
         # default processing
         if not self.has_queue:
             if not self._transition_queue:
@@ -391,20 +432,22 @@ class AsyncMachine(Machine):
             else:
                 raise MachineError("Attempt to process events synchronously while transition queue is not empty!")
 
-        self._transition_queue.append(trigger)
+        self._transition_queue_dict[model].append(trigger)
         # another entry in the queue implies a running transition; skip immediate execution
-        if len(self._transition_queue) > 1:
+        if len(self._transition_queue_dict[model]) > 1:
             return True
 
-        # execute as long as transition queue is not empty ToDo: not tested!
-        while self._transition_queue:
+        while self._transition_queue_dict[model]:
             try:
-                await self._transition_queue[0]()
-                self._transition_queue.popleft()
+                await self._transition_queue_dict[model][0]()
             except Exception:
                 # if a transition raises an exception, clear queue and delegate exception handling
-                self._transition_queue.clear()
+                self._transition_queue_dict[model].clear()
                 raise
+            try:
+                self._transition_queue_dict[model].popleft()
+            except KeyError:
+                return True
         return True
 
 
@@ -442,9 +485,13 @@ class HierarchicalAsyncMachine(HierarchicalMachine, AsyncMachine):
         for key, value in _state_tree.items():
             if value:
                 with self(key):
-                    res[key] = await self._trigger_event(_model, _trigger, value, *args, **kwargs)
+                    tmp = await self._trigger_event(_model, _trigger, value, *args, **kwargs)
+                    if tmp is not None:
+                        res[key] = tmp
             if not res.get(key, None) and _trigger in self.events:
-                res[key] = await self.events[_trigger].trigger(_model, self, *args, **kwargs)
+                tmp = await self.events[_trigger].trigger(_model, self, *args, **kwargs)
+                if tmp is not None:
+                    res[key] = tmp
         return None if not res or all([v is None for v in res.values()]) else any(res.values())
 
 
@@ -505,7 +552,7 @@ class AsyncTimeout(AsyncState):
 
         """
         timer_task = self.runner.get(id(event_data.model), None)
-        if timer_task is not None:
+        if timer_task is not None and not timer_task.done():
             timer_task.cancel()
         await super().exit(event_data)
 
@@ -531,7 +578,6 @@ class AsyncTimeout(AsyncState):
         _LOGGER.debug("%sTimeout state %s. Processing callbacks...", event_data.machine.name, self.name)
         await event_data.machine.callbacks(self.on_timeout, event_data)
         _LOGGER.info("%sTimeout state %s processed.", event_data.machine.name, self.name)
-        self.runner[id(event_data.model)] = None
 
     @property
     def on_timeout(self):
@@ -544,3 +590,15 @@ class AsyncTimeout(AsyncState):
     def on_timeout(self, value):
         """ Listifies passed values and assigns them to on_timeout."""
         self._on_timeout = listify(value)
+
+
+class _DictionaryMock(dict):
+
+    def __setitem__(self, key, item):
+        self._value = item
+
+    def __getitem__(self, key):
+        return self._value
+
+    def __repr__(self):
+        return repr("{{'*': {0}}}".format(self._value))
