@@ -192,13 +192,17 @@ class NestedState(State):
 
     def scoped_enter(self, event_data, scope=[]):
         self._scope = scope
-        self.enter(event_data)
-        self._scope = []
+        try:
+            self.enter(event_data)
+        finally:
+            self._scope = []
 
     def scoped_exit(self, event_data, scope=[]):
         self._scope = scope
-        self.exit(event_data)
-        self._scope = []
+        try:
+            self.exit(event_data)
+        finally:
+            self._scope = []
 
     @property
     def name(self):
@@ -231,6 +235,7 @@ class NestedTransition(Transition):
         src_name_path = event_data.source_path
         if src_name_path == dst_name_path:
             root = src_name_path[:-1]  # exit and enter the same state
+            dst_name_path = dst_name_path[-1:]
         else:
             root = []
             while dst_name_path and src_name_path and src_name_path[0] == dst_name_path[0]:
@@ -246,9 +251,7 @@ class NestedTransition(Transition):
         else:
             new_states, enter_partials = {}, []
 
-        for key in scoped_tree:
-            del scoped_tree[key]
-
+        scoped_tree.clear()
         for new_key, value in new_states.items():
             scoped_tree[new_key] = value
             break
@@ -281,7 +284,8 @@ class NestedTransition(Transition):
             q = []
             prefix = prefix_path
             scoped_tree = new_states
-            initial_states = [event_data.machine.scoped.states[i] for i in listify(event_data.machine.scoped.initial)]
+            initial_names = [i.name if hasattr(i, 'name') else i for i in listify(event_data.machine.scoped.initial)]
+            initial_states = [event_data.machine.scoped.states[n] for n in initial_names]
             while True:
                 event_data.scope = prefix
                 for state in initial_states:
@@ -368,7 +372,12 @@ class HierarchicalMachine(Machine):
         initial_name = getattr(models[0], self.model_attribute)
         if hasattr(initial_name, 'name'):
             initial_name = initial_name.name
-        initial_states = self._resolve_initial(models, initial_name.split(self.state_cls.separator))
+        # initial states set by add_model or machine might contain initial states themselves.
+        if isinstance(initial_name, string_types):
+            initial_states = self._resolve_initial(models, initial_name.split(self.state_cls.separator))
+        # when initial is set to a (parallel) state, we accept it as it is
+        else:
+            initial_states = initial_name
         for mod in models:
             self.set_state(initial_states, mod)
             if hasattr(mod, 'to'):
@@ -378,6 +387,15 @@ class HierarchicalMachine(Machine):
                 to_func = partial(self.to_state, mod)
                 setattr(mod, 'to', to_func)
 
+    @property
+    def initial(self):
+        """ Return the initial state. """
+        return self._initial
+
+    @initial.setter
+    def initial(self, value):
+        self._initial = self._recursive_initial(value)
+
     def add_ordered_transitions(self, states=None, trigger='next_state',
                                 loop=True, loop_includes_initial=True,
                                 conditions=None, unless=None, before=None,
@@ -414,9 +432,11 @@ class HierarchicalMachine(Machine):
         ignore = self.ignore_invalid_triggers if ignore_invalid_triggers is None else ignore_invalid_triggers
 
         for state in listify(states):
-            if isinstance(state, Enum) and isinstance(state.value, EnumMeta):
-                state = {'name': state, 'children': state.value}
-
+            if isinstance(state, Enum):
+                if isinstance(state.value, EnumMeta):
+                    state = {'name': state, 'children': state.value}
+                elif isinstance(state.value, dict):
+                    state = dict(name=state, **state.value)
             if isinstance(state, string_types):
                 if remap is not None and state in remap:
                     return
@@ -436,6 +456,10 @@ class HierarchicalMachine(Machine):
                     self.states[new_state.name] = new_state
                     self._init_state(new_state)
             elif isinstance(state, Enum):
+                if self.state_cls.separator in state.name:
+                    raise ValueError("State '{0}' contains '{1}' which is used as state name separator. "
+                                     "Consider changing the NestedState.separator to avoid this issue."
+                                     "".format(state.name, self.state_cls.separator))
                 if remap is not None and state.name in remap:
                     return
                 new_state = self._create_state(state, on_enter=on_enter, on_exit=on_exit,
@@ -562,14 +586,45 @@ class HierarchicalMachine(Machine):
                 ordered_states.extend(self.get_nested_state_names())
         return ordered_states
 
-    def get_nested_triggers(self, dest=None):
-        if dest:
-            triggers = _super(HierarchicalMachine, self).get_triggers(dest)
+    def get_nested_transitions(self, trigger="", src_path=None, dest_path=None):
+        if src_path and dest_path:
+            src = self.state_cls.separator.join(src_path)
+            dest = self.state_cls.separator.join(dest_path)
+            transitions = _super(HierarchicalMachine, self).get_transitions(trigger, src, dest)
+            if len(src_path) > 1 and len(dest_path) > 1:
+                with self(src_path[0]):
+                    transitions.extend(self.get_nested_transitions(trigger, src_path[1:], dest_path[1:]))
+        elif src_path:
+            src = self.state_cls.separator.join(src_path)
+            transitions = _super(HierarchicalMachine, self).get_transitions(trigger, src, "*")
+            if len(src_path) > 1:
+                with self(src_path[0]):
+                    transitions.extend(self.get_nested_transitions(trigger, src_path[1:], None))
+        elif dest_path:
+            dest = self.state_cls.separator.join(dest_path)
+            transitions = _super(HierarchicalMachine, self).get_transitions(trigger, "*", dest)
+            if len(dest_path) > 1:
+                for state_name in self.states:
+                    with self(state_name):
+                        transitions.extend(self.get_nested_transitions(trigger, None, dest_path[1:]))
+        else:
+            transitions = _super(HierarchicalMachine, self).get_transitions(trigger, "*", "*")
+            for state_name in self.states:
+                with self(state_name):
+                    transitions.extend(self.get_nested_transitions(trigger, None, None))
+        return transitions
+
+    def get_nested_triggers(self, src_path=None):
+        if src_path:
+            triggers = _super(HierarchicalMachine, self).get_triggers(self.state_cls.separator.join(src_path))
+            if len(src_path) > 1 and src_path[0] in self.states:
+                with self(src_path[0]):
+                    triggers.extend(self.get_nested_triggers(src_path[1:]))
         else:
             triggers = list(self.events.keys())
-        for state in self.states.values():
-            with self(state.name):
-                triggers.extend(self.get_nested_triggers())
+            for state_name in self.states:
+                with self(state_name):
+                    triggers.extend(self.get_nested_triggers())
         return triggers
 
     def get_state(self, state, hint=None):
@@ -608,25 +663,69 @@ class HierarchicalMachine(Machine):
                 res.append(self.get_state(state))
         return res
 
+    def get_transitions(self, trigger="", source="*", dest="*", delegate=False):
+        """ Return the transitions from the Machine.
+        Args:
+            trigger (str): Trigger name of the transition.
+            source (str, State or Enum): Limits list to transitions from a certain state.
+            dest (str, State or Enum): Limits list to transitions to a certain state.
+            delegate (Optional[bool]): If True, consider delegations to parents of source
+        """
+        with self():
+            source_path = [] if source == "*" \
+                else source.split(self.state_cls.separator) if isinstance(source, string_types) \
+                else self._get_enum_path(source) if isinstance(source, Enum) \
+                else self._get_state_path(source)
+            dest_path = [] if dest == "*" \
+                else dest.split(self.state_cls.separator) if isinstance(dest, string_types) \
+                else self._get_enum_path(dest) if isinstance(dest, Enum) \
+                else self._get_state_path(dest)
+            matches = self.get_nested_transitions(trigger, source_path, dest_path)
+            # only consider delegations when source_path contains a nested state (len > 1)
+            if delegate is False or len(source_path) < 2:
+                return matches
+            source_path.pop()
+            while source_path:
+                matches.extend(self.get_transitions(trigger,
+                                                    source=self.state_cls.separator.join(source_path),
+                                                    dest=dest))
+                source_path.pop()
+            return matches
+
     def get_triggers(self, *args):
         """ Extends transitions.core.Machine.get_triggers to also include parent state triggers. """
-        # add parents to state set
         triggers = []
         with self():
             for state_name in args:
                 state_path = state_name.split(self.state_cls.separator)
-                root = state_path[0]
-                while state_path:
+                if len(state_path) > 1:  # we only need to check substates when 'state_name' refers to a substate
+                    with self(state_path[0]):
+                        triggers.extend(self.get_nested_triggers(state_path[1:]))
+                while state_path:  # check all valid transitions for parent states
                     triggers.extend(_super(HierarchicalMachine, self).get_triggers(self.state_cls.separator.join(state_path)))
-                    with self(root):
-                        triggers.extend(self.get_nested_triggers(self.state_cls.separator.join(state_path)))
                     state_path.pop()
         return triggers
 
+    def has_trigger(self, trigger, state=None):
+        """ Check whether an event/trigger is known to the machine
+        Args:
+            trigger (str): Event/trigger name
+            state (optional[NestedState]): Limits the recursive search to this state and its children
+        Returns:
+            bool: True if event is known and False otherwise
+        """
+
+        state = state or self
+        return trigger in state.events or any([self.has_trigger(trigger, sta) for sta in state.states.values()])
+
     def is_state(self, state_name, model, allow_substates=False):
         current_name = getattr(model, self.model_attribute)
         if allow_substates:
-            return current_name.startswith(state_name.name if hasattr(state_name, 'name') else state_name)
+            if isinstance(current_name, Enum):
+                current_name = self.state_cls.separator.join(self._get_enum_path(current_name))
+            if isinstance(state_name, Enum):
+                state_name = self.state_cls.separator.join(self._get_enum_path(state_name))
+            return current_name.startswith(state_name)
         return current_name == state_name
 
     def on_enter(self, state_name, callback):
@@ -758,6 +857,16 @@ class HierarchicalMachine(Machine):
                     return res
         return []
 
+    def _get_state_path(self, state, prefix=[]):
+        if state in self.states.values():
+            return prefix + [state.name]
+        for name in self.states:
+            with self(name):
+                res = self._get_state_path(state, prefix=prefix + [name])
+                if res:
+                    return res
+        return []
+
     def _check_event_result(self, res, model, trigger):
         if res is None:
             state_names = getattr(model, self.model_attribute)
@@ -767,7 +876,12 @@ class HierarchicalMachine(Machine):
                 ignore = state.ignore_invalid_triggers if state.ignore_invalid_triggers is not None \
                     else self.ignore_invalid_triggers
                 if not ignore:
-                    raise MachineError(msg)
+                    # determine whether a MachineError (valid event but invalid state) ...
+                    if self.has_trigger(trigger):
+                        raise MachineError(msg)
+                    # or AttributeError (invalid event) is appropriate
+                    else:
+                        raise AttributeError("Do not know event named '%s'." % trigger)
             _LOGGER.warning(msg)
             res = False
         return res
@@ -782,10 +896,7 @@ class HierarchicalMachine(Machine):
         Returns:
             bool: True if a transitions has been conducted or the trigger event has been queued.
         """
-        try:
-            return self.trigger_event(model, trigger_name, *args, **kwargs)
-        except MachineError:
-            raise AttributeError("Do not know event named '%s'." % trigger_name)
+        return self.trigger_event(model, trigger_name, *args, **kwargs)
 
     def _has_state(self, state, raise_error=False):
         """ This function
@@ -825,6 +936,24 @@ class HierarchicalMachine(Machine):
             for substate in self.states.values():
                 self._init_state(substate)
 
+    def _recursive_initial(self, value):
+        if isinstance(value, string_types):
+            path = value.split(self.state_cls.separator, 1)
+            if len(path) > 1:
+                state_name, suffix = path
+                # make sure the passed state has been created already
+                _super(HierarchicalMachine, self.__class__).initial.fset(self, state_name)
+                with self(state_name):
+                    self.initial = suffix
+                    self._initial = state_name + self.state_cls.separator + self._initial
+            else:
+                _super(HierarchicalMachine, self.__class__).initial.fset(self, value)
+        elif isinstance(value, (list, tuple)):
+            return [self._recursive_initial(v) for v in value]
+        else:
+            _super(HierarchicalMachine, self.__class__).initial.fset(self, value)
+        return self._initial[0] if isinstance(self._initial, list) and len(self._initial) == 1 else self._initial
+
     def _resolve_initial(self, models, state_name_path, prefix=[]):
         if state_name_path:
             state_name = state_name_path.pop(0)
@@ -853,7 +982,11 @@ class HierarchicalMachine(Machine):
         for key, value in _state_tree.items():
             if value:
                 with self(key):
-                    res[key] = self._trigger_event(_model, _trigger, value, *args, **kwargs)
-            if not res.get(key, None) and _trigger in self.events:
-                res[key] = self.events[_trigger].trigger(_model, self, *args, **kwargs)
+                    tmp = self._trigger_event(_model, _trigger, value, *args, **kwargs)
+                    if tmp is not None:
+                        res[key] = tmp
+            if res.get(key, False) is False and _trigger in self.events:
+                tmp = self.events[_trigger].trigger(_model, self, *args, **kwargs)
+                if tmp is not None:
+                    res[key] = tmp
         return None if not res or all(v is None for v in res.values()) else any(res.values())
